using System;
using SdlDotNet;
using System.Drawing;

namespace Physics.Entities.Physics
{
    /* represents any object/thing that will be drawn to the screen
     */
    abstract class PhysicsE : Entity
    {
        // PhysicsEntity's are affected by forces
        protected Vector2 Force = new Vector2(0, 0);

        public float RVelocity
        { get { return RVelo; } }

        // each Entity has a current Rotation, RVelocity and Torque
        public float Rotation = 0F;
        protected float RVelo = 0F;
        protected float Torque = 0F;

        public bool StaticCollisions = false;
        public static bool antiAliasingOn = false;

        // a value holding the mass of this Entity
        public float Mass = 1F;

        // Coefficient of Restitution
        // due to the CoR penatration quick fix, energy is gained when CoR = 1, or close to it
        public float CoR = 0.6F;

        // All subclasses should have a property that calculates the RMass using the current mass
        // this depends on the type of the object and hence is virtual
        public virtual float RMass
        {
            get { return 0; }
        }

        /// <summary>
        /// an Update method for a PEntity.
        /// applies any forces and torques and updates the entities velocities and locations.
        /// </summary>
        /// <param name="timeStep">The time since the last update</param>
        public override void Update(float timeStep)
        {
            Vector2 Accel = Force / Mass;
            float RAccel = Torque / RMass;

            Position = PositionAt(timeStep);
            Rotation = RotationAt(timeStep);

            Velo = VelocityAt(timeStep);
            RVelo = RVelocityAt(timeStep);

            Force *= 0;
            Torque = 0;
        }

        /// <summary>
        /// Takes a given force from a location and applies it to Torque and Force
        /// </summary>
        /// <param name="Force">The force vector</param>
        /// <param name="Location">The location from which the given force is affecting this entity</param>
        public void ApplyForce(Vector2 Force)
        {
            ApplyForce(Force, Position);
        }
        /// <summary>
        /// Takes a given force from a location and applies it to Torque and Force
        /// </summary>
        /// <param name="Force">The force vector</param>
        /// <param name="Location">The location from which the given force is affecting this entity</param>
        public void ApplyForce(Vector2 Force, Vector2 Location)
        {
            this.Force += Force;

            if (Location.Equals(Position))
                return;

            Vector2 v = (Location - Position).Unit;
            Torque += v.Dot(Force.Perp);
        }

        /// <summary>
        /// Takes a given impluse from a location and applies it to RVelocity and Velocity
        /// </summary>
        /// <param name="Impluse">The impluse vector</param>
        /// <param name="Location">The location from which the given impluse is affecting this entity</param>
        public void ApplyImpulse(Vector2 Impulse)
        {
            ApplyForce(Impulse, Position);
        }
        /// <summary>
        /// Takes a given impluse from a location and applies it to RVelocity and Velocity
        /// </summary>
        /// <param name="Impluse">The impluse vector</param>
        /// <param name="Location">The location from which the given impluse is affecting this entity</param>
        public void ApplyImpulse(Vector2 Impulse, Vector2 Location)
        {
            Velo += Impulse;

            if (Location.Equals(Position))
                return;

            Vector2 v = (Location - Position).Unit;
            RVelo += v.Dot(Impulse.Perp);
        }

        /// <summary>
        /// Calculates the Velocity of a given point on this object.
        /// Does NOT garentied this point is inside this entity.
        /// </summary>
        /// <param name="Point">The point at which to calculate velocity</param>
        /// <returns>The velocity of the given point</returns>
        public Vector2 GetVelocityOfPoint(Vector2 ScreenPoint)
        {
            return Velo + (ScreenPoint - Position) * RVelocity;
        }

        public Vector2 PositionAt(float timeStep)
        {
            Vector2 Accel = Force / Mass;
            return Accel / 2 * timeStep * timeStep + Velo * timeStep + Position;
        }
        public float RotationAt(float timeStep)
        {
            float RAccel = Torque / RMass;
            return RAccel / 2 * timeStep * timeStep + RVelo * timeStep + Rotation;
        }
        public Vector2 VelocityAt(float timeStep)
        {
           Vector2 Accel = Force / Mass;
           return Accel * timeStep + Velo;
        }
        public float RVelocityAt(float timeStep)
        {
            float RAccel = Torque / RMass;
            return RAccel * timeStep + RVelo;
        }
    }
}