using System;
using System.Drawing;

namespace Physics
{
    class Vector2
    {
        public float X;
        public float Y;

        public float Angel
        {
            get
            {
                if (X < 0 && Y < 0)
                {
                    return (float)(Math.Atan(Y / X) + Math.PI);
                }
                else
                {
                    return (float)Math.Atan(Y / X);
                }
            }
        }
        public float Length
        {
            get
            {
                return (float)Math.Sqrt(Dot(this));
            }
        }

        public Vector2 Unit
        {
            get
            {
                return Divide(Length);
            }
        }
        public Vector2 Perp
        {
            get
            {
                return new Vector2(-Y, X);
            }
        }

        public Point Point
        {
            get
            {
                return new Point((int)X, (int)Y);
            }
        }

        public Vector2(float x, float y)
        {
            X = x;
            Y = y;
        }
        public Vector2(Point p)
        {
            X = p.X;
            Y = p.Y;
        }

        public Vector2 Clone()
        {
            return new Vector2(X, Y);
        }

        #region operators

        public Vector2 Add(Vector2 vec)
        {
            return new Vector2(X + vec.X, Y + vec.Y);
        }
        public Vector2 Subtract(Vector2 vec)
        {
            return new Vector2(X - vec.X, Y - vec.Y);
        }
        public Vector2 Divide(float c)
        {
            return new Vector2(X / c, Y / c);
        }
        public Vector2 Multiply(float c)
        {
            return new Vector2(X * c, Y * c);
        }

        public static Vector2 operator +(Vector2 vec1, Vector2 vec2)
        {
            return vec1.Add(vec2);
        }
        public static Vector2 operator -(Vector2 vec1, Vector2 vec2)
        {
            return vec1.Subtract(vec2);
        }

        public static Vector2 operator / (Vector2 vec, float c)
        {
            return vec.Divide(c);
        }
        public static Vector2 operator * (Vector2 vec, float c)
        {
            return vec.Multiply(c);
        }
        public static Vector2 operator *(float c, Vector2 vec)
        {
            return vec.Multiply(c);
        }
        #endregion

        // takes the Dot Product of two vectors
        public float Dot(Vector2 vector2)
        {
            return X * vector2.X + Y * vector2.Y;
        }

        public float ProjectionCoeff(Vector2 ontoY)
        {
            float top = Dot(ontoY);
            float bot = ontoY.Dot(ontoY);
            return top / bot;
        }
        public Vector2 Projection(Vector2 ontoY)
        {
            return ProjectionCoeff(ontoY) * ontoY;
        }

        // return a vector of min length from the point to the line 
        public Vector2 VectorToLine(float x1, float y1, float x2, float y2)
        {
            float lineSize = (float) Math.Sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
            if (lineSize == 0)
                return new Vector2(x1 - X, y1 - Y);

            float u = ((X - x1) * (x2 - x1) + (Y - y1) * (y2 - y1)) / (lineSize * lineSize);

            if (u < 0)
                return new Vector2(x1 - X, y1 - Y);
            else if (u > 1)
                return new Vector2(x2 - X, y2 - Y);
            else
            {
                float ix = x1 + u * (x2 - x1);
                float iy = y1 + u * (y2 - y1);
                return new Vector2(ix - X, iy - Y);
            }
        }

        public void Rotate(float rads)
        {
            float t = (float)(Math.Cos(rads) * X + Math.Sin(rads) * Y);
            Y = (float)(Math.Cos(rads) * Y - Math.Sin(rads) * X);
            X = t;
        }

        public override string ToString()
        {
            return "Vector2  X: " + X + ", Y: " + Y;
        }

        public void Zero()
        {
            X = 0;
            Y = 0;
        }

        // taken from http://local.wasp.uwa.edu.au/~pbourke/geometry/lineline2d/
        public static Vector2 LineIntersection(Vector2 aP1, Vector2 aP2, Vector2 bP1, Vector2 bP2)
        {
            float x1 = aP1.X;
            float x2 = aP2.X;
            float x3 = bP1.X;
            float x4 = bP2.X;

            float y1 = aP1.Y;
            float y2 = aP2.Y;
            float y3 = bP1.Y;
            float y4 = bP2.Y;

            float denom = (y4 - y3) * (x2 - x1) - (x4 - x3) * (y2 - y1);

            if (denom == 0)
                return null; // coincident or parallel

            float ua = ((x4 - x3) * (y1 - y3) - (y4 - y3) * (x1 - x3)) / denom;
            float ub = ((y4 - y3) * (x1 - x3) - (x4 - x3) * (y1 - y3)) / denom;

            if (0 <= ua && ua <= 1 && 0 <= ub && ub <= 1)
            {
                float x = x1 + ua * (x2 - x1);
                float y = y1 + ua * (y2 - y1);

                return new Vector2(x, y);
            }

            return null;
        }
    }
}
